/*
Use: Use get the data from API.
*/

import 'dart:io';
import 'package:dio/adapter.dart';
import 'package:dio/dio.dart';
import 'dio_error_entity.dart';
import 'dio_return_error.dart';
import 'dio_method.dart';

const _apiTypeCollection = {
  DioMethod.get: 'get',
  DioMethod.post: 'post',
  DioMethod.delete: 'delete',
  DioMethod.put: 'put'
};

typedef HttpSuccessCallback<T> = void Function(T data);
typedef HttpFailureCallback = void Function(DioErrorEntity error);
typedef ProgressCallback = void Function(int count, int total);

CancelToken cancelToken = CancelToken();

class DioHelper {
  late Dio _dio;
  DioHelper(this._dio);

  Future request<T>(String baseUrl,
      {DioMethod dioMethod = DioMethod.get,
      String? path = '',
      dynamic data,
      Map<String, dynamic>? queryParameter,
      Options? options,
      required HttpSuccessCallback<T> success,
      required HttpFailureCallback error,
      Map<String, dynamic>? header}) async {
    try {
      BaseOptions baseOptions = BaseOptions(
          baseUrl: baseUrl,
          receiveDataWhenStatusError: false,
          responseType: ResponseType.json,
          connectTimeout: 60000,
          receiveTimeout: 3000,
          headers: header);

      _dio = Dio(baseOptions)..interceptors.addAll(logInterceptor());

      (_dio.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate =
          (client) {
        client.badCertificateCallback =
            (X509Certificate cert, String host, int port) {
          return true;
        };
      };

      //GET FEATURE DATA
      var response = await _dio.request(
        path!,
        data: data,
        onSendProgress: (received, total) {
          if (total != -1) {
            print((received / total * 100).toStringAsFixed(0) + '%');
          }
        },
        queryParameters: queryParameter,
        options: Options(method: _apiTypeCollection[dioMethod]),
      );

      success(response.data);
    } on DioError catch (e) {
      error(dioReturnError(e));
    }
  }

  List<Interceptor> logInterceptor() {
    return [LogInterceptor(requestBody: true, responseBody: true)];
  }

  void cancelRequests(CancelToken token) {
    token.cancel('cancelled');
  }
}
