//RETURN ERROR
import 'package:dio/dio.dart';
import 'dio_error_entity.dart';

DioErrorEntity dioReturnError(DioError error) {
  switch (error.type) {
    case DioErrorType.cancel:
      {
        return const DioErrorEntity(code: -1, message: 'Request cancellation');
      }
    case DioErrorType.connectTimeout:
      {
        return const DioErrorEntity(code: -1, message: 'Connection timed out');
      }
    case DioErrorType.sendTimeout:
      {
        return const DioErrorEntity(code: -1, message: 'Request timed out');
      }
    case DioErrorType.receiveTimeout:
      {
        return const DioErrorEntity(code: -1, message: 'Response timeout');
      }
    case DioErrorType.other:
      return const DioErrorEntity(
          code: 0, message: 'Check your internet connection');

    case DioErrorType.response:
      {
        try {
          int? errCode = error.response?.statusCode;
          switch (errCode) {
            case 400:
              return DioErrorEntity(
                  code: errCode!, message: 'Request syntax error');

            case 403:
              return DioErrorEntity(
                  code: errCode!, message: 'Server refused to execute');

            case 404:
              return DioErrorEntity(
                  code: errCode!, message: 'Data not available!');

            case 405:
              return DioErrorEntity(
                  code: errCode!, message: 'Request method is forbidden');

            case 500:
              return DioErrorEntity(
                  code: errCode!, message: 'Server internal error');

            case 502:
              return DioErrorEntity(code: errCode!, message: 'Invalid request');

            case 503:
              return DioErrorEntity(code: errCode!, message: 'Server is down');

            case 505:
              return DioErrorEntity(
                  code: errCode!,
                  message: 'Does not support HTTP protocol request');

            default:
              return DioErrorEntity(code: errCode!, message: 'Unknown mistake');
          }
        } on Exception catch (_) {
          return const DioErrorEntity(code: -1, message: 'Unknown error');
        }
      }

    default:
      return DioErrorEntity(code: -1, message: error.message);
  }
}
